@@ -1,8 +1,7 @@
 #import "notify.h"
 
 // getBundleIdentifier(app_name: &str) -> "com.apple.Terminal"
-NSString* getBundleIdentifier(NSString* appName)
-{
+NSString* getBundleIdentifier(NSString* appName) {
     NSString* findString = [NSString stringWithFormat:@"get id of application \"%@\"", appName];
     NSAppleScript* findScript = [[NSAppleScript alloc] initWithSource:findString];
     NSAppleEventDescriptor* resultDescriptor = [findScript executeAndReturnError:nil];
@@ -10,27 +9,26 @@ NSString* getBundleIdentifier(NSString* appName)
 }
 
 // setApplication(new_bundle_identifier: &str) -> Result<()>
-BOOL setApplication(NSString* newbundleIdentifier)
-{
-    if (LSCopyApplicationURLsForBundleIdentifier((CFStringRef)newbundleIdentifier, NULL) != NULL)
-    {
-        fakeBundleIdentifier = newbundleIdentifier;
-        return YES;
+// invariant: this function should be called at most once and before `sendNotification`
+BOOL setApplication(NSString* newbundleIdentifier) {
+    @autoreleasepool {
+        if (!installNSBundleHook()) {
+            return NO;
+        }
+        if (LSCopyApplicationURLsForBundleIdentifier((CFStringRef)newbundleIdentifier, NULL) != NULL) {
+            [fakeBundleIdentifier release]; // Release old value - nil is ok
+            fakeBundleIdentifier = newbundleIdentifier;
+            [newbundleIdentifier retain]; // Retain new value - it outlives this scope
+
+            return YES;
+        }
+        return NO;
     }
-    return NO;
 }
 
 // sendNotification(title: &str, subtitle: &str, message: &str, options: Notification) -> NotificationResult<()>
-NSDictionary* sendNotification(NSString* title, NSString* subtitle, NSString* message, NSDictionary* options)
-{
-    @autoreleasepool
-    {
-        if (!installNSBundleHook())
-        {
-            // TODO: Could potentially have different error messages
-            return @{@"error" : @""};
-        }
-
+NSDictionary* sendNotification(NSString* title, NSString* subtitle, NSString* message, NSDictionary* options) {
+    @autoreleasepool {
         // For a list of available notification options, see https://developer.apple.com/documentation/foundation/nsusernotification?language=objc
 
         NSUserNotificationCenter* notificationCenter = [NSUserNotificationCenter defaultUserNotificationCenter];
@@ -46,21 +44,22 @@ NSDictionary* sendNotification(NSString* title, NSString* subtitle, NSString* me
 
         // Basic text
         userNotification.title = title;
-        if (![subtitle isEqualToString:@""])
-        {
+        if (![subtitle isEqualToString:@""]) {
             userNotification.subtitle = subtitle;
         }
         userNotification.informativeText = message;
 
         // Notification sound
-        if (options[@"sound"] && ![options[@"sound"] isEqualToString:@""] && ![options[@"sound"] isEqualToString:@"_mute"])
-        {
-            userNotification.soundName = options[@"sound"];
+        if (options[@"sound"] && ![options[@"sound"] isEqualToString:@""]) {
+            if ([options[@"sound"] isEqualToString:@"NSUserNotificationDefaultSoundName"]) {
+                userNotification.soundName = NSUserNotificationDefaultSoundName;
+            } else {
+                userNotification.soundName = options[@"sound"];
+            }
         }
 
         // Delivery Date/Schedule
-        if (options[@"deliveryDate"] && ![options[@"deliveryDate"] isEqualToString:@""])
-        {
+        if (options[@"deliveryDate"] && ![options[@"deliveryDate"] isEqualToString:@""]) {
             ncDelegate.keepRunning = YES;
             double deliveryDate = [options[@"deliveryDate"] doubleValue];
             NSDate* scheduleTime = [NSDate dateWithTimeIntervalSince1970:deliveryDate];
@@ -70,71 +69,62 @@ NSDictionary* sendNotification(NSString* title, NSString* subtitle, NSString* me
         }
 
         // Main Actions Button (defaults to "Show")
-        if (options[@"mainButtonLabel"] && ![options[@"mainButtonLabel"] isEqualToString:@""])
-        {
+        if (options[@"mainButtonLabel"] && ![options[@"mainButtonLabel"] isEqualToString:@""]) {
             ncDelegate.keepRunning = YES;
             userNotification.actionButtonTitle = options[@"mainButtonLabel"];
             userNotification.hasActionButton = 1;
+        } else {
+            userNotification.hasActionButton = 0;
         }
 
         // Dropdown actions
-        if (options[@"actions"] && ![options[@"actions"] isEqualToString:@""])
-        {
+        if (options[@"actions"] && ![options[@"actions"] isEqualToString:@""]) {
             ncDelegate.keepRunning = YES;
             [userNotification setValue:@YES forKey:@"_showsButtons"];
 
             NSArray* myActions = [options[@"actions"] componentsSeparatedByString:@","];
 
-            if (myActions.count > 1)
-            {
+            if (myActions.count > 1) {
                 [userNotification setValue:@YES forKey:@"_alwaysShowAlternateActionMenu"];
                 [userNotification setValue:myActions forKey:@"_alternateActionButtonTitles"];
             }
         }
 
         // Close/Other button (defaults to "Cancel")
-        if (options[@"closeButtonLabel"] && ![options[@"closeButtonLabel"] isEqualToString:@""])
-        {
+        if (options[@"closeButtonLabel"] && ![options[@"closeButtonLabel"] isEqualToString:@""]) {
             ncDelegate.keepRunning = YES;
             [userNotification setValue:@YES forKey:@"_showsButtons"];
             userNotification.otherButtonTitle = options[@"closeButtonLabel"];
         }
 
         // Reply to the notification with a text field
-        if (options[@"response"] && ![options[@"response"] isEqualToString:@""])
-        {
+        if (options[@"response"] && ![options[@"response"] isEqualToString:@""]) {
             ncDelegate.keepRunning = YES;
             userNotification.hasReplyButton = 1;
             userNotification.responsePlaceholder = options[@"mainButtonLabel"];
         }
 
         // Change the icon of the app in the notification
-        if (options[@"appIcon"] && ![options[@"appIcon"] isEqualToString:@""])
-        {
+        if (options[@"appIcon"] && ![options[@"appIcon"] isEqualToString:@""]) {
             NSImage* icon = getImageFromURL(options[@"appIcon"]);
             // replacement app icon
             [userNotification setValue:icon forKey:@"_identityImage"];
             [userNotification setValue:@(false) forKey:@"_identityImageHasBorder"];
         }
         // Change the additional content image
-        if (options[@"contentImage"] && ![options[@"contentImage"] isEqualToString:@""])
-        {
+        if (options[@"contentImage"] && ![options[@"contentImage"] isEqualToString:@""]) {
             userNotification.contentImage = getImageFromURL(options[@"contentImage"]);
         }
 
         // If set to asynchronous, do not wait for actions
-        if (options[@"asynchronous"] && [options[@"asynchronous"] isEqualToString:@"yes"])
-        {
+        if (options[@"asynchronous"] && [options[@"asynchronous"] isEqualToString:@"yes"]) {
             ncDelegate.keepRunning = NO;
         }
 
         // Send or schedule notification
-        if (isScheduled)
-        {
+        if (isScheduled) {
             [notificationCenter scheduleNotification:userNotification];
-        }
-        else
-        {
+        } else {
             [notificationCenter deliverNotification:userNotification];
         }
 
@@ -142,8 +132,7 @@ NSDictionary* sendNotification(NSString* title, NSString* subtitle, NSString* me
 
         // TODO: Issue #4 mentions an issue with multithreading, perhaps there could be an overall "synchronous" option (instead of deliveryDate's synchronous section)
         // Loop/wait for a user action if needed
-        while (ncDelegate.keepRunning)
-        {
+        while (ncDelegate.keepRunning) {
             [[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.1]];
         }
 
